/*
0+1 viscous hydro solver for general equations of state


by

Paul Romatschke

version 1.0 December 2011

Type ./vh0 to get syntax

Example:

./vh0 1.0 0.5 0.15 0.5 3.0 0.0 idEOS.dat 0.0 10000 0.001

This code solves causal viscous hydro equations derived in

R. Baier, P. Romatschke, D.T. Son, A. Starinets, \
M. Stephanov, arXiv:0712.2451,
(JHEP 0804:100,2008).

If you use this code or some part of it, please be so kind as
to add a reference to this article. Thank you.

 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fstream>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>



using namespace std;

const double fmtoGeVinv=5.06773116;


//for the equation of state
char* eosname;
long int length;
double *eoT4,*cs2i,*poT4,*Ti;
//splines -- for equation of state
gsl_interp_accel *pacc,*Tacc,*cs2acc,*eacc;
gsl_spline *pspline,*Tspline,*cs2spline,*espline;
//to know where to stop interpolation
double lowestE;



double t_neu, t_alt, phi_neu, phi_alt, eps_neu, eps_alt, tau;
//double a=(8*exp(2*log(pi))/45);
double a=8/(3*M_PI*M_PI);
double ETAS=1.0;
double COEFF=0.5;
double deltatau=0.001;
double endtau=40.;
double L1COEFF=0.0;


void Tneu (double);
void epsneu (double);
void Phineu (double);
void loadeos();
double pofe(double mye);

double T(double);
double eta(double);
double taupi(double);
double lambda1oe2(double);


int main(int argc, char* argv[])
{

  if (argc<9)
    {
      printf("syntax: vh0 [tau_0 fm] [T_0 GeV] [T_f GeV] [eta/s] [(eps+P)tau_pi/eta/2] [(eps+P)lambda1/eta^2] [eosfilename]\n");
      printf("optional arguments: arg #8 [pi^eta_eta initial GeV]\n");
      printf("optional arguments: arg #9 [print Tevolution skipping number]\n");
      printf("optional arguments: arg #10 [delta tau GeV]\n");
      return 1;
    }
  else
    {
     

      double tstart=atof(argv[1]);
      double Tstart=atof(argv[2]);
      double Tstop=atof(argv[3]);
      ETAS=atof(argv[4]);
      COEFF=atof(argv[5]);
      L1COEFF=atof(argv[6]);
      eosname=argv[7];
      double Phistart=0;
      int debug=0;

      if (argc>8)
	Phistart=atof(argv[8]);
      if (argc>9)
	debug=atoi(argv[9]);
      if (argc>10)
	deltatau=atof(argv[10]);

      printf("Input: \n-----------------------------------------------------\n");
      printf("eta/s = %f\t T (eps+P)tau_pi/eta/2=%f\t (eps+P)lambda1/eta^2=%f\n",ETAS,COEFF,L1COEFF);
      printf("eosfile name: %s\n",eosname);

      loadeos();
      
      eps_alt=gsl_spline_eval(espline,Tstart,eacc);

      printf("Starting evolution at tau0=%f fm and T0=%f GeV eps0=%f GeV^4\n",tstart,Tstart,eps_alt);
  

      

      fstream ausg;
      ausg.open("Tevolution.dat",ios::out);

      char buffer[255];

      int zaehler=0;

      if (debug)
	printf("tau [fm/c]\tT[GeV]\n");
      for (tau=tstart*fmtoGeVinv, t_alt=Tstart, phi_alt=Phistart; t_alt>=Tstop; tau=tau+deltatau)
	{
	  epsneu(tau);
	  Tneu (tau);
	  Phineu (tau);
	  zaehler++;
	  
	  //printf("Tneu=%f Tneu(eps)=%f\n",t_neu,T(eps_neu));

	  t_neu=T(eps_neu);
	  if (debug)
	    if ((zaehler-1)%debug==0)
	      printf("%g\t\t%g\n",tau/fmtoGeVinv, T(eps_alt));
	    
	 
	  if ((zaehler-1)%(debug+1)==0)
	    {
	      sprintf(buffer,"%g\t\t%g\t%g\n",tau/fmtoGeVinv, T(eps_alt), T(eps_alt)/(Tstart*pow(tstart*fmtoGeVinv/tau,1./3.)));
	      ausg<<buffer;
	    }
	  
	  
	  t_alt=t_neu;
	  eps_alt=eps_neu;
	  phi_alt=phi_neu;
	}
      ausg.close();
      
      return 0;
    }
}



void epsneu (double tau2)
{ 
  eps_neu=deltatau*(phi_alt-(eps_alt+pofe(eps_alt)))/tau2+eps_alt;
}

void Tneu (double tau2)
{ 
  t_neu=(-t_alt/(3*tau2)+phi_alt/(12*a*exp(3*log(t_alt))*tau2))*deltatau+t_alt;
}

void Phineu (double tau3)
{
  phi_neu=phi_alt+deltatau*(4.*eta(eps_alt)/(3*taupi(eps_alt)*tau3)-phi_alt/taupi(eps_alt)-4*phi_alt/(3*tau3)-lambda1oe2(eps_alt)/(2*taupi(eps_alt))*phi_alt*phi_alt);
			    
			    //phi_neu=(-(phi_alt*t_alt)/(2*COEFF*etas)+(8*a*exp(4*log(t_alt)))/(3*COEFF*tau3)
			    //	   -4/(3*tau3)*phi_alt-L1/(32*COEFF*M_PI*etas*etas*a*t_alt*t_alt*t_alt)*phi_alt*phi_alt)*deltatau+phi_alt;
}


void loadeos()
{
  fstream eosf;

  //load specified eos file

   eosf.open(eosname, ios::in);

   if (eosf.is_open())
    {
      eosf >> length;
      printf("Read length of %li\n",length);

      

      eoT4 = new double[length];
      cs2i = new double[length];
      poT4 = new double[length];
      Ti = new double[length];

      for (int i=1;i<=length;i++)
        {
          eosf >> Ti[i-1];
          eosf >> eoT4[i-1];
          eosf >> poT4[i-1];
          eosf >> cs2i[i-1];
        }

      eosf.close();
    }
  else
    printf("Could not open EOS file\n");

  //interpolate
  pacc=gsl_interp_accel_alloc (); 
  Tacc=gsl_interp_accel_alloc (); 
  cs2acc=gsl_interp_accel_alloc (); 
  eacc=gsl_interp_accel_alloc (); 
  pspline=gsl_spline_alloc (gsl_interp_cspline, length);
  Tspline=gsl_spline_alloc (gsl_interp_cspline, length);
  cs2spline=gsl_spline_alloc (gsl_interp_cspline, length);
  espline=gsl_spline_alloc (gsl_interp_cspline, length);
  
  double warrx[length];
  double warry[length];

  for (int i=0;i<length;i++)
    {
      warrx[i]=eoT4[i]*Ti[i]*Ti[i]*Ti[i]*Ti[i];
      warry[i]=poT4[i]*Ti[i]*Ti[i]*Ti[i]*Ti[i];
    }

  lowestE=warry[0];

  gsl_spline_init (pspline,warrx,warry,length);
  
  for (int i=0;i<length;i++)
    {
      warry[i]=Ti[i];
    }  
  gsl_spline_init (Tspline,warrx,warry,length);

  gsl_spline_init (espline,warry,warrx,length);
  
  for (int i=0;i<length;i++)
    {
      warry[i]=cs2i[i];
    }  
  gsl_spline_init (cs2spline,warrx,warry,length);

}

//returns pressure as function of energy density
double pofe(double mye)
{
  double temp=0;
  
  double phys=mye;
  
  if (phys>lowestE)
    {
      temp=gsl_spline_eval(pspline,phys,pacc);
    }
  else
    {
      temp=gsl_spline_eval(cs2spline,lowestE,cs2acc);
      temp*=phys;
    }

  return temp;
}

double T(double eps)
{
  return gsl_spline_eval(Tspline,eps,Tacc);
}

double eta(double eps)
{
   double temp=ETAS*(eps+pofe(eps))/T(eps);
   return temp;
}

double taupi(double eps)
{
  double temp=COEFF*2*ETAS/T(eps);
  return temp;
}

//lambda1 over eta^2
double lambda1oe2(double eps)
{
  double temp=L1COEFF/(eps+pofe(eps));
  return temp;
}
